from __future__ import annotations
from dataclasses import dataclass
from typing import List
from enum import Enum
import csv
import json
from jinja2 import Environment, FileSystemLoader, select_autoescape
from pronounceable import PronounceableWord, generate_word
import io
import segno
from base64 import b64encode
from exam import Exam

CSV_DELIMITERS = [ ',', ';' ]  # if we need to detect delimiters since they are different in the provided CSV
DEFAULT_PASSWORD_GENERATOR = lambda : PronounceableWord().length(8, 12)

@dataclass
class Student:
    fullname: str
    birth_date: str
    scei: int
    password: str

class Students:
    def __init__(self):
        self.students = []

    @classmethod
    def from_csv(cls, csv_filename : str, detectDelimiter:bool=False, delimiter:str=';') -> Students:
        with open(csv_filename, newline='', encoding='cp1252') as csvfile:
            # try to detect CSV delimiter
            header = csvfile.readline()
            if detectDelimiter:
                delimiter = None
                for potential_delimiter in CSV_DELIMITERS:
                    if header.count(potential_delimiter) == 2:
                        delimiter = potential_delimiter
                        break
                if delimiter is None:
                    raise Exception('Failed to detect CSV delimiter.')

            student_reader = csv.reader(csvfile, delimiter=delimiter, quotechar='|')
            students = Students()
            students.students = [ Student(row[0], row[1], row[2], password=None) for row in student_reader ]
        return students

    def generate_passwords(self, password_generator=DEFAULT_PASSWORD_GENERATOR):
        for student in self.students:
            student.password = password_generator()

    def as_json(self, json_filename : str):
        with open(json_filename, 'w') as outfile:
            json.dump(self.students, outfile, default=lambda o: o.__dict__, indent=2, ensure_ascii=False)

    def coupons_as_html(self, exam : Exam) -> str:
        for s in self.students:
            qr = segno.make_qr(json.dumps({'scei' : s.scei, 'fullname' : s.fullname }))
            buff = io.BytesIO()
            qr.save(buff, kind='png', scale=100)
            s.qr_code = b64encode(buff.getvalue()).decode("utf-8") 
            buff.close()

        env = Environment(
            loader = FileSystemLoader('../templates'),
            autoescape = select_autoescape(['html', 'xml'])
        )
        template = env.get_template('coupons.tmpl')

        html = template.render(exam=exam, students=self.students)

        # remove tempory field qr_code
        for s in self.students:
            del s.qr_code
        return html